#include "burp.h"
#include "prog.h"
#include "msg.h"
#include "lock.h"
#include "rs_buf.h"
#include "handy.h"
#include "asyncio.h"
#include "zlibio.h"
#include "counter.h"
#include "dpth.h"
#include "sbuf.h"

void init_sbuf(struct sbuf *sb)
{
	sb->cmd='e';
	sb->path=NULL;
	sb->plen=0;
	sb->linkto=NULL;
	sb->llen=0;
	sb->sendpath=0;
	sb->datapth=NULL;
	sb->senddatapth=0;
	sb->statbuf=NULL;
	sb->slen=0;
	sb->sendstat=0;

	memset(&(sb->rsbuf), 0, sizeof(sb->rsbuf));
	sb->sigjob=NULL;
	sb->infb=NULL;
	sb->outfb=NULL;
	sb->sigfp=NULL;
	sb->sendendofsig=0;

	sb->receivedelta=0;

	sb->fp=NULL;
	sb->zp=NULL;

	sb->endfile=NULL;
	sb->elen=0;
}

void free_sbuf(struct sbuf *sb)
{
	if(sb->path) free(sb->path);
	if(sb->linkto) free(sb->linkto);
	if(sb->datapth) free(sb->datapth);
	if(sb->statbuf) free(sb->statbuf);
	if(sb->sigjob) rs_job_free(sb->sigjob);
	if(sb->infb) rs_filebuf_free(sb->infb);
	if(sb->outfb) rs_filebuf_free(sb->outfb);
	gzclose_fp(&sb->sigfp);
	close_fp(&sb->fp);
	gzclose_fp(&sb->zp);
	init_sbuf(sb);
	if(sb->endfile) free(sb->endfile);
}

int cmd_is_file(char cmd)
{
	if(cmd=='f') return 1;
	return 0;
}

int sbuf_is_file(struct sbuf *sb)
{
	return cmd_is_file(sb->cmd);
}

int cmd_is_link(char cmd)
{
	if(cmd=='l' || cmd=='L') return 1;
	return 0;
}

int sbuf_is_link(struct sbuf *sb)
{
	return cmd_is_link(sb->cmd);
}

int cmd_is_endfile(char cmd)
{
	if(cmd=='x') return 1;
	return 0;
}

int sbuf_is_endfile(struct sbuf *sb)
{
	return cmd_is_endfile(sb->cmd);
}

static int do_sbuf_fill(FILE *fp, gzFile zp, struct sbuf *sb, int phase1)
{
	int ars;
	free_sbuf(sb);
	if((ars=async_read_stat(fp, zp, &(sb->statbuf), &(sb->slen),
		&(sb->statp), &(sb->datapth))))
			return ars;
	if((ars=async_read_fp(fp, zp, &(sb->cmd), &(sb->path), &(sb->plen))))
		return ars;
	//sb->path[sb->plen]='\0'; sb->plen--; // avoid new line
	if(sbuf_is_link(sb))
	{
		char cmd;
		if((ars=async_read_fp(fp, zp, &cmd,
			&(sb->linkto), &(sb->llen))))
				return ars;
	//	sb->linkto[sb->llen]='\0'; sb->llen--; // avoid new line
		if(!cmd_is_link(cmd))
		{
			logp("got non-link cmd after link cmd: %c %s\n",
				cmd, sb->linkto);
			return -1;
		}
	}
	else if(!phase1 && sbuf_is_file(sb))
	{
		char cmd;
		if((ars=async_read_fp(fp, zp, &cmd,
			&(sb->endfile), &(sb->elen))))
				return ars;
		if(!cmd_is_endfile(cmd))
		{
			logp("got non-endfile cmd after file: %c %s\n",
				cmd, sb->endfile);
			return -1;
		}
	}
	return 0;
}

int sbuf_fill(FILE *fp, gzFile zp, struct sbuf *sb)
{
	return do_sbuf_fill(fp, zp, sb, 0);
}

int sbuf_fill_phase1(FILE *fp, gzFile zp, struct sbuf *sb)
{
	return do_sbuf_fill(fp, zp, sb, 1);
}

static int sbuf_to_fp(struct sbuf *sb, FILE *mp)
{
	if(sb->path)
	{
		if(sb->datapth)
			send_msg_fp(mp, 't', sb->datapth, strlen(sb->datapth));
		send_msg_fp(mp, 'r', sb->statbuf, sb->slen);
		send_msg_fp(mp, sb->cmd, sb->path, sb->plen);
		if(sb->linkto)
			send_msg_fp(mp, sb->cmd, sb->linkto, sb->llen);
		if(sbuf_is_file(sb))
			send_msg_fp(mp, 'x', sb->endfile, sb->elen);
	}
	return 0;
}

static int sbuf_to_zp(struct sbuf *sb, gzFile zp)
{
	if(sb->path)
	{
		if(sb->datapth)
			send_msg_zp(zp, 't', sb->datapth, strlen(sb->datapth));
		send_msg_zp(zp, 'r', sb->statbuf, sb->slen);
		send_msg_zp(zp, sb->cmd, sb->path, sb->plen);
		if(sb->linkto)
			send_msg_zp(zp, sb->cmd, sb->linkto, sb->llen);
		if(sbuf_is_file(sb))
			send_msg_zp(zp, 'x', sb->endfile, sb->elen);
	}
	return 0;
}

int sbuf_to_manifest(struct sbuf *sb, FILE *mp, gzFile zp)
{
	if(mp) return sbuf_to_fp(sb, mp);
	if(zp) return sbuf_to_zp(sb, zp);
	logp("No valid file pointer given to sbuf_to_manifest()\n");
	return -1;
}
